/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/
import React, { useEffect, useState } from 'react';
import moment from "moment";
import styled from "styled-components"
//Material ui componets
import { withStyles } from '@material-ui/core/styles';
import TableBody from '@material-ui/core/TableBody';
import Grid from "@material-ui/core/Grid";
import Paper from "@material-ui/core/Paper";
import MenuItem from "@material-ui/core/MenuItem";
import TableRow from "@material-ui/core/TableRow";
import TextField from "@material-ui/core/TextField";
import TableCell from '@material-ui/core/TableCell';
import RemoveCircleOutlineRoundedIcon from '@material-ui/icons/RemoveCircleOutlineRounded';
import VisibilityIcon from '@material-ui/icons/Visibility';
import { Button } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded'
import EmailRoundedIcon from '@material-ui/icons/EmailRounded';
//Local files
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import { apiDomain } from '../../../../env';
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader from "../../../Components/Components/PageHeader"
//Services
import { getRequest, putRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { Url } from '../../../Filters';
//routers
import { Link } from 'react-router-dom';

//Image Import
import { noAvatar } from "ImportImages.js";

const StyledTableCell = withStyles((theme) => ({
  head: {
    backgroundColor: theme.palette.common.black,
    color: theme.palette.common.white,
  },
  body: {
    fontSize: 14,
  },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
  root: {
    '&:nth-of-type(odd)': {
      backgroundColor: theme.palette.action.hover,
    },
  },
}))(TableRow);

const BlockedUsers = () => {
  const ADD_ONE_LENGHT = [''];
  const WINDOW_WIDTH = window.innerWidth

  const [invertList, setInvertList] = useState(false)
  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete 
  const [items, setItems] = useState([]); //Necessary to consult the API, data
  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false)
  const [stateOpt, setStateOpt] = useState(1)
  const [currPage, setCurrPage] = useState(0)

  const [snackInfo, setSnackInfo] = useState({
    message: '',
    icon: '',
    open: false,
    color: '',
  })

  const StateOptions = [
    {
      name: 1,
      value: "Semanal"
    },
    {
      name: 2,
      value: "Permanente"
    }
  ];

  // Handle snack infos
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color
    })
  }

  const handleChange = (event) => {
    const type = event.target.value
    setStateOpt(type)
  }

  const BlockStatus = (status) => {
    switch (status) {
      case "blocked":
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: "#FF8C00",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            SEMANAL
          </Paper>
        );
      case "banished":
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: "red",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            PERMANENTE
          </Paper>
        );
      default:
        return (
          <Paper
            style={{
              textAlign: "center",
              padding: "0.5em",
              backgroundColor: "#797D7F",
              fontWeight: "500",
              color: "#FFFAFA",
            }}
          >
            Não avaliado
          </Paper>
        );
    }
  };

  const ReactiveUser = (userId, index) => {
    putRequest(
      `/users/${userId}/reactivate_user`,
      {},
      (data) => {
        if (data.errors)
          HandleSnack('Erro!', true, 'warning', '#FA8072')
        else {
          HandleSnack('Usuário foi reativado com sucesso!', true, 'success', '#228B22')
          removeItemFromList(index)
          setCurrPage(0)
        }
      },
      (error) => {
        HandleSnack('Erro!', true, 'warning', '#FA8072')
      }
    )
  }

  const removeItemFromList = (index) => {
    const copyList = [...items]
    copyList.splice(index, 1)
    setItems(copyList)
  }

  const DisplayDate = (date) => {
    const convertedData = moment.utc(date);
    return moment(convertedData)
      .format("LLL")
      .toString();
  };

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      Url('users', `"state" : ${stateOpt}`, currPage, invertList ? "ASC" : 'DESC'),

      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, stateOpt, invertList])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {

    //Words in the top part of the table
    const topTable = ['ID', 'ESTADO', 'NAME', 'EMAIL', 'BLOQUEADO EM', 'AÇÕES'];
    if (WINDOW_WIDTH <= 899) {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />
          <MobilePageHeader
            title="Lista de usuários bloqueados"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Inverter lista",
                isLoading: false,
                func: () => {
                  cleanArrayAndInvert()
                },
              },
            ]}
          >
            <Grid item>
              <TextField
                select
                label="Estado de bloqueio"
                onChange={handleChange}
                helperText="Por favor, selecione uma das opções"
              >
                {StateOptions.map((option, index) => (
                  <MenuItem
                    key={option.value}
                    value={option.name}
                    name={option.value}
                  >
                    {option.value}
                  </MenuItem>
                ))}
              </TextField>
            </Grid>
          </MobilePageHeader>

          <div style={{ height: '2em' }}></div>

          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton>
                <Button
                  key={new Date().toISOString() + row.created_at}
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </Button>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    key={new Date().toISOString() + row.created_at}
                    title={row.name}
                    subtitle={row.id}
                    backColor={"#e81f4f"}
                    avatar={
                      <img
                        src={row.avatar ? apiDomain + row.avatar : noAvatar}
                        alt="user avatar"
                        style={{
                          height: "100%",
                          width: "100%",
                          borderRadius: "50%",
                        }}
                      />
                    }
                    href={`/admin/user?id=${row.id}`}
                    reset={() => {
                    }}
                    data={
                      [
                        {
                          title: "Email",
                          subtitle: row.email ?
                            <Link to={`/admin/sendEmail?email=${row.email}`} style={{ textDecoration: 'none' }}>
                              <Button
                                variant='text'
                                color='primary'
                                startIcon={<EmailRoundedIcon />}
                              >
                                {row.email}
                              </Button>
                            </Link> : null

                        },
                        {
                          title: "Estado",
                          subtitle: BlockStatus(row.state)
                        },
                        {
                          title: "Ações rápidas",
                          subtitle: <Button
                            style={{ width: "100%", marginBottom: "0.5em" }}
                            variant="contained"
                            color="secondary"
                            startIcon={<RemoveCircleOutlineRoundedIcon />}
                            onClick={() => ReactiveUser(row.id, index)}
                          >
                            Desbloquear
                                                    </Button>
                        }
                      ]
                    }
                  />
                  <div style={{ height: "0.5em" }} />
                </>
              )
          )}
        </div>
      )
    }
    else {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />
          <PageHeader
            title="Lista de usuários bloqueados"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
            ]}
          >
            <Grid item>
              <TextField
                select
                label="Estado de bloqueio"
                onChange={handleChange}
                helperText="Por favor, selecione uma das opções"
              >
                {StateOptions.map((option, index) => (
                  <MenuItem
                    key={option.value}
                    value={option.name}
                    name={option.value}
                  >
                    {option.value}
                  </MenuItem>
                ))}
              </TextField>
            </Grid>
          </PageHeader>

          <div style={{ height: '2em' }}></div>

          <TableData
            top={topTable}
            onIconPressed={cleanArrayAndInvert}
          >
            <TableBody>
              {items.map((row, index) => (
                index === items.length - 1 ?
                  <StyledTableRow key={new Date().toISOString() + row.created_at}>
                    {/* Button to load more data */}
                    <StyledTableCell>
                      <Button
                        color='primary'
                        variant='text'
                        disabled={isLoadingMoreItems}
                        startIcon={<AddRoundedIcon />}
                        onClick={() => {
                          setCurrPage(currPage + 1)
                        }}
                      >
                        {
                          isLoadingMoreItems ? <CircularProgress size={24} /> : 'Carregar mais itens'
                        }
                      </Button>
                    </StyledTableCell>
                  </StyledTableRow>

                  :

                  <StyledTableRow key={new Date().toISOString() + index}>
                    <StyledTableCell component="th" scope="row">{row.id}</StyledTableCell>
                    <StyledTableCell align="right">
                      {BlockStatus(row.state)}
                    </StyledTableCell>
                    <StyledTableCell align="right">{row.name}</StyledTableCell>
                    <StyledTableCell align="right">
                      {
                        row.email ?
                          <Link to={`/admin/sendEmail?email=${row.email}`} style={{ textDecoration: 'none' }}>
                            <Button
                              variant='text'
                              color='primary'
                              startIcon={<EmailRoundedIcon />}
                            >
                              {row.email}
                            </Button>
                          </Link> : null
                      }
                    </StyledTableCell>
                    <StyledTableCell align="right">
                      {DisplayDate(row.suspended_at)}
                    </StyledTableCell>
                    <StyledTableCell align="right">
                      <Button
                        style={{ width: "100%", marginBottom: "0.5em" }}
                        variant="contained"
                        color="secondary"
                        startIcon={<RemoveCircleOutlineRoundedIcon />}
                        onClick={() => ReactiveUser(row.id, index)}
                      >
                        Desbloquear
                                            </Button>
                      <Link to={`/admin/user?id=${row.id}`}>
                        <Button
                          style={{ width: "100%" }}
                          variant="contained"
                          color="primary"
                          startIcon={<VisibilityIcon />}
                        >
                          Visualizar
                                                </Button>
                      </Link>
                    </StyledTableCell>
                  </StyledTableRow>
              ))}
            </TableBody>
          </TableData>
        </div>
      )
    }
  }
}


export default BlockedUsers;

const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
`