/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState } from 'react';
//Material ui componets
import { withStyles } from '@material-ui/core/styles';
import TableBody from '@material-ui/core/TableBody';
import Paper from "@material-ui/core/Paper";
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import IconButton from '@material-ui/core/IconButton';
import VisibilityIcon from '@material-ui/icons/Visibility';
import { Button } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded';
import DeleteRoundedIcon from '@material-ui/icons/DeleteRounded';
//Local files'
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader from "../../../Components/Components/PageHeader"
//Services
import AlertDialog from "../../../Components/Components/AlertDialog";
import { Url } from '../../../Filters';
import { DeleteFilter } from '../../../Filters';
import { getRequest, deleteRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//routers
import { Link, useHistory } from 'react-router-dom';
import StarRoundedIcon from "@material-ui/icons/StarRounded";
import styled from "styled-components"

const StyledTableCell = withStyles((theme) => ({
  head: {
    backgroundColor: theme.palette.common.black,
    color: theme.palette.common.white,
  },
  body: {
    fontSize: 14,
  },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
  root: {
    '&:nth-of-type(odd)': {
      backgroundColor: theme.palette.action.hover,
    },
  },
}))(TableRow);

const Ratings = () => {
  const WINDOW_WIDTH = window.innerWidth
  const ADD_ONE_LENGHT = [""];

  const router = useHistory()

  const [invertList, setInvertList] = useState(false)
  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete 
  const [items, setItems] = useState([]); //Necessary to consult the API, data
  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false)
  const [currPage, setCurrPage] = useState(0)
  const [openAlertDialog, setOpenAlertDialog] = useState(false); //controlls the state od alert dialog 
  const [deleteItem, setDeleteItem] = useState({}); //Delete Item
  const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);

  const [snackInfo, setSnackInfo] = useState({
    message: '',
    icon: '',
    open: false,
    color: '',
  })

  // Handle snack infos
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color
    })
  }

  //handle Delete
  async function DeleteHandler() {
    const id = deleteItem.id;
    HandleStateAlertDialog(null);
    deleteRequest(
      DeleteFilter("ratings", id),
      (data) => {
        if (data.errors)
          HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
        else {
          HandleSnack(
            "O rating foi deletada com sucesso",
            true,
            "success",
            "#228B22"
          );
          setCurrPage(0)
          HandleStateCircularProgress(null);
          removeItemFromList(id)
        }
      },
      (error) => {
        HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
        HandleStateCircularProgress(null);
      }
    )
  }

  const removeItemFromList = (itemId) => {
    let index = -1;
    for (let i = 0; i < items.length; i++) {
      const element = items[i];
      if (element.id === itemId) {
        index = i
        break
      }
    }
    if (index !== -1) {
      const cpyItems = [...items]
      cpyItems.splice(index, 1)
      setItems(cpyItems)
    }
  }

  const HandleStateCircularProgress = (i) => {
    setIsLoadingToDelete(i);
  };

  const HandleStateAlertDialog = (i) => {
    const obj = { ...items[i] };
    setDeleteItem(obj);
    setOpenAlertDialog(!openAlertDialog);
  };

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      Url("ratings", "", currPage, invertList ? "ASC" : "DESC"),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, invertList])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {

    //Words in the top part of the table
    const topTable = ['ID', 'NOME', 'DESCRIÇÃO', 'VISUALIZAR', 'DELETAR'];
    if (WINDOW_WIDTH <= 800) {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
          <MobilePageHeader
            title="Lista de ratings"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/CreateRating')
                },
                icon: <AddRoundedIcon />
              }
            ]}
          >
          </MobilePageHeader>

          <div style={{ height: '2em' }}></div>

          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton key={new Date().toISOString() + row.created_at}>
                <Button
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </Button>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    key={new Date().toISOString() + row.created_at}
                    title={row.name}
                    subtitle={row.id}
                    backColor={"#00bcd4"}
                    avatar={<StarRoundedIcon />}
                    href={`/admin/Rating?rating=${row.id}`}
                    reset={() => {

                    }}
                    data={
                      [
                        {
                          title: "Descrição",
                          subtitle: row.description
                        },
                        {
                          title: "Deletar",
                          subtitle:
                            <Button
                              variant="contained"
                              color="secondary"
                              onClick={() => {
                                HandleStateAlertDialog(index);
                                HandleStateCircularProgress(index);
                              }}
                              startIcon={<DeleteRoundedIcon />}
                            >
                              Deletar
                            </Button>
                        }
                      ]
                    }
                  />
                  <div style={{ height: "0.5em" }} />
                </>
              )
          )}
        </div>
      )
    }
    else {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />
          <PageHeader
            title="Lista de ratings"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/CreateRating')
                },
                icon: <AddRoundedIcon />
              }
            ]}
          >
          </PageHeader>

          <div style={{ height: '2em' }}></div>

          <TableData
            top={topTable}
            onIconPressed={cleanArrayAndInvert}
          >
            <TableBody>
              {items.map((row, index) => (
                index === items.length - 1 ?
                  <StyledTableRow key={new Date().toISOString() + row.created_at}>
                    {/* Button to load more data */}
                    <StyledTableCell>
                      <Button
                        color='primary'
                        variant='text'
                        disabled={isLoadingMoreItems}
                        startIcon={<AddRoundedIcon />}
                        onClick={() => {
                          setCurrPage(currPage + 1)
                        }}
                      >
                        {
                          isLoadingMoreItems ? <CircularProgress size={24} /> : 'Carregar mais itens'
                        }
                      </Button>
                    </StyledTableCell>
                  </StyledTableRow>

                  :

                  <StyledTableRow key={new Date().toISOString() + index}>
                    <StyledTableCell component="th" scope="row">{row.id}</StyledTableCell>
                    <StyledTableCell align="right">{row.name}</StyledTableCell>
                    <StyledTableCell align="right">{row.description}</StyledTableCell>
                    <StyledTableCell align="right">
                      <Link to={`/admin/Rating?rating=${row.id}`}>
                        <IconButton>
                          <VisibilityIcon style={{ fill: '#00bcd4' }} />
                        </IconButton>
                      </Link>
                    </StyledTableCell>
                    <StyledTableCell align="right">
                      {isLoadingToDelete === index ? (
                        <CircularProgress size={24} color="primary" />
                      ) : (
                          <IconButton
                            onClick={() => {
                              HandleStateAlertDialog(index);
                              HandleStateCircularProgress(index);
                            }}
                          >
                            <DeleteRoundedIcon style={{ fill: "#FF0000" }} />
                          </IconButton>
                        )}
                    </StyledTableCell>
                  </StyledTableRow>
              ))}
            </TableBody>
          </TableData>

          {/* This alert will be displayed if the user click to delete an institution */}
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
        </div>
      )
    }
  }
}


export default Ratings;

const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
`

