/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { Component } from "react";
import "./CardsAnimation.css";
import { Container } from "react-grid-system";
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardActions from "@material-ui/core/CardActions";
import IconButton from "@material-ui/core/IconButton";
import Typography from "@material-ui/core/Typography";
import FavoriteIcon from "@material-ui/icons/Favorite";
import styled from 'styled-components';
import Rating from "@material-ui/lab/Rating";
import StarBorderIcon from "@material-ui/icons/StarBorder";
import { apiDomain } from "../env";
import AddIcon from "@material-ui/icons/CreateNewFolder";
import Options from "./CardOptions";
import Video from "@material-ui/icons/OndemandVideo";

//Image Import
import { Animacao } from "ImportImages.js";
import { Apresentacao } from "ImportImages.js";
import { AplicativoMovel } from "ImportImages.js";
import { Audio } from "ImportImages.js";
import { Empty } from "ImportImages.js";
import { Imagem } from "ImportImages.js";
import { Infografico } from "ImportImages.js";
import { Jogo } from "ImportImages.js";
import { LivroDigital } from "ImportImages.js";
import { Mapa } from "ImportImages.js";
import { Outros } from "ImportImages.js";
import { SoftwareEducacional } from "ImportImages.js";
import { Texto } from "ImportImages.js";
import { video } from "ImportImages.js";
import { noAvatar } from "ImportImages.js";

const types = [
  { label: "Animação", thumb: Animacao },
  { label: "Apresentação", thumb: Apresentacao },
  { label: "Aplicativo Movel", thumb: AplicativoMovel },
  { label: "Áudio", thumb: Audio },
  { label: "Vazio", thumb: Empty },
  { label: "Imagem", thumb: Imagem },
  { label: "Gráfico", thumb: Infografico },
  { label: "Jogo", thumb: Jogo },
  { label: "Livro", thumb: LivroDigital },
  { label: "Livro digital", thumb: LivroDigital },
  { label: "Mapa", thumb: Mapa },
  { label: "Outros", thumb: Outros },
  { label: "Software Educacional", thumb: SoftwareEducacional },
  { label: "Software", thumb: SoftwareEducacional },
  { label: "Texto", thumb: Texto },
  { label: "Vídeo", thumb: video }
];
const slideStyle = {
  height: "50px",
  width: "50px",
  backgroundColor: "white",
  borderRadius: "50%"
};
const publisherStyle = {
  color: "white",
  paddinLeft: "30px",
  fontSize: "15px",
};

class ResourceCard extends Component {
  constructor(props) {
    super(props);
    this.state = {
      thumbnail: null,
      userStyle: "animatiOff",
      userAvatar: null
    };
  }
  renderType(type) {
    if (!this.state.thumbnail) {
      if (!this.props.thumbnail) {
        const aux = types.find(function (element) { return element.label === type });
        this.setState({ thumbnail: aux.thumb });
      }
      else {
        this.setState({ thumbnail: `${apiDomain}` + this.props.thumbnail });
      }
    }
    // {/*if (!this.state.thumbnail && !this.props.thumbnail ) {
    //   var aux = types.find(function(element){ return element.label === type});
    //   this.setState({thumbnail: aux.thumb});
    // }
    // if (!this.state.thumbnail && this.props.thumbnail ) {
    //   this.setState({thumbnail: apiDomain + this.props.thumbnail});
    // }*/}
  }
  onHover() {
    this.setState({ userStyle: "animatiOn" });
  }
  onIddle() {
    this.setState({ userStyle: "animatiOff" });
  }
  defAvatar() {
    if (!this.state.userAvatar && !this.props.avatar.length) {
      this.setState({ userAvatar: noAvatar });
    }
    if (!this.state.userAvatar && this.props.avatar) {
      this.setState({ userAvatar: apiDomain + this.props.avatar });
    }
  }
  render() {
    this.renderType(this.props.type);
    this.defAvatar();
    return (
      <Card
        onMouseOver={this.onHover.bind(this)}
        onMouseOut={this.onIddle.bind(this)}
      >
        <Container
          style={{
            padding: "0px",
            height: "175px",
            backgroundColor: "#ff7f00"
          }}
        >
          <div className={this.state.userStyle}>
            <img style={slideStyle} src={this.state.userAvatar} alt="Avatar" />
            <StyledPublisher style={publisherStyle}>
              Enviado por: <br />
              {this.props.author}
            </StyledPublisher>
          </div>
          <img
            src={this.state.thumbnail}
            style={{ height: "100%" }}
            alt="thumbnail do recurso"
          />
        </Container>
        <CardContent
          style={{ height: "60%", textAlign: "left", paddingBottom: "0px" }}
        >
          <Typography
            variant="body2"
            color="textSecondary"
            component="p"
            style={{ height: "45px", overflow: "hidden", fontSize: "0.8em" }}
          >
            {this.props.name}
          </Typography>
        </CardContent>
        <Container style={{ textAlign: "left" }}>
          <Rating
            name="customized-empty"
            value={this.props.rating * 10}
            precision={0.5}
            readOnly
            emptyIcon={<StarBorderIcon fontSize="inherit" />}
          />
        </Container>
        <CardActions
          style={{ justifyContent: "space-between", padding: "0px" }}
        >
          {this.props.type === "Vídeo" ? (
            <Video style={{ color: "#ff7f00" }} />
          ) : (
              <br />
            )}
          <IconButton aria-label="Favoritar" size="small">
            <FavoriteIcon />
          </IconButton>
        </CardActions>
        <CardActions
          style={{
            borderTop: "1px solid #e5e5e5",
            justifyContent: "space-between"
          }}
        >
          <IconButton aria-label="Guardar" size="small">
            <AddIcon />
          </IconButton>
          <Options />
        </CardActions>
      </Card>
    );
  }
}

const StyledPublisher = styled(Typography)`
  display: -webkit-box;
  -webkit-line-clamp: 2;
  -webkit-box-orient: vertical;
  overflow: hidden;
`

export default ResourceCard;
